unit uswitch;

{$mode objfpc}{$H+}

interface

uses
  Graphics, Classes, SysUtils, Controls;

type

  { TSwitch }

  TSwitch = class(TCustomControl)
  private
    FChecked: Boolean;
    FColorON: TColor;
    FColorOFF: TColor;
    FCaptionON: String;
    FCaptionOFF: String;
    FSliderWidth: Integer;
    FOnChange: TNotifyEvent;
    function IsSliderWidthStored: Boolean;
    procedure SetCaptionOFF(const AValue: String);
    procedure SetCaptionON(const AValue: String);
    procedure SetChecked(const AValue: Boolean);
    procedure SetColorOFF(const AValue: TColor);
    procedure SetColorON(const AValue: TColor);
    procedure SetSliderWidth(const AValue: Integer);
  protected
    procedure DblClick; override;
    procedure DoAutoAdjustLayout(const AMode: TLayoutAdjustmentPolicy;
      const AXProportion, AYProportion: Double); override;
    procedure DoChange; virtual;
    procedure DoEnter; override;
    procedure DoExit; override;
    procedure DrawAll;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property CaptionOFF: String read FCaptionON write SetCaptionOFF;
    property CaptionON: String read FCaptionOFF Write SetCaptionON;
    property Checked: Boolean read FChecked write SetChecked default false;
    property ColorOFF: TColor read FColorOFF write SetColorOFF default clBlack;
    property ColorON: TColor read FColorON write SetColorON default clGreen;
//    property SliderWidth: Integer read FSliderWidth write SetSliderWidth default 24;
    property SliderWidth: Integer read FSliderWidth write SetSliderWidth stored IsSliderWidthStored;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

implementation

uses
  LCLIntf, LCLType;

const
  DEFAULT_SLIDER_WIDTH = 24;

constructor TSwitch.Create(AOwner: TComponent);
begin
  inherited;
  Width := 60;
  Height := 30;
  Color := clWindow;
  TabStop := true;
  FChecked := false;
  FColorON := clGreen;
  FColorOFF := clBlack;
  FCaptionON := 'ON';
  FCaptionOFF := 'OFF';
  FSliderWidth := Scale96ToFont(DEFAULT_SLIDER_WIDTH);
end;

procedure TSwitch.DblClick;
begin
  inherited;
  Checked := not FChecked;
end;

procedure TSwitch.DoChange;
begin
  if Assigned(FOnChange) then FOnChange(self);
end;

procedure TSwitch.DoAutoAdjustLayout(
  const AMode: TLayoutAdjustmentPolicy;
  const AXProportion, AYProportion: Double);
begin
  inherited;
  if AMode in [lapAutoAdjustWithoutHorizontalScrolling,
               lapAutoAdjustForDPI]
  then begin
    DisableAutosizing;
    try
      if not IsSliderWidthStored then
        FSliderWidth := Round(FSliderWidth * AXProportion);
    finally
      EnableAutoSizing;
    end;
  end;
end;

procedure TSwitch.DoEnter;
begin
  inherited;
  Invalidate;
end;

procedure TSwitch.DoExit;
begin
  inherited;
  Invalidate;
end;

procedure TSwitch.DrawAll;
var
  ROuter: TRect;
  RSlider: TRect;
  RText: TRect;
  s: String;
  txtStyle: TTextStyle;
begin
  // Draw outer rectangle
  ROuter := Rect(0, 0, Width, Height);
  Canvas.Brush.Style := bsSolid;
  Canvas.Brush.Color := Color;
  Canvas.Pen.Style := psSolid;
  Canvas.Pen.Color := clCaptionText;
  Canvas.Pen.Width := 1;
  Canvas.Rectangle(Router);

  // Draw slider
  RSlider := ROuter;
  InflateRect(RSlider, -4, -4);
  if FChecked then begin
    // ON --> slider is at the right
    RSlider.Left := RSlider.Right - FSliderWidth;
    Canvas.Brush.Color := FColorON;
  end else begin
    // OFF --> slider is at the left
    RSlider.Right := RSlider.Left + FSliderWidth;
    Canvas.Brush.Color := FColorOFF;
  end;
  Canvas.Rectangle(RSlider);

  // Draw caption
  Canvas.Font.Assign(Font);
  if FChecked then begin
    // ON --> text at left of slider
    Rtext := Rect(0, 0, RSlider.Left, Height);
    s := FCaptionON;
  end else begin
    // OFF --> text at right of slider
    RText := Rect(RSlider.Right, 0, Width, Height);
    s := FCaptionOFF;
  end;
  txtStyle := Canvas.TextStyle;
  txtStyle.Alignment := taCenter;
  txtStyle.Layout := tlCenter;
  Canvas.TextRect(RText, RText.Left, RText.Top, s, txtStyle);

  // Draw focus rectangle
  if Focused then begin
    InflateRect(RSlider, 2, 2);
    Canvas.Pen.Mode := pmXOR;
    Canvas.Pen.Style := psDot;
    Canvas.Pen.Color := clWhite;
    Canvas.Pen.Cosmetic := false;
    Canvas.Brush.Style := bsClear;
    Canvas.Rectangle(Rslider);
    Canvas.Pen.Mode := pmCopy;
  end;
end;

function TSwitch.IsSliderWidthStored: Boolean;
begin
  Result := FSliderWidth <> Scale96ToFont(DEFAULT_SLIDER_WIDTH);
end;

procedure TSwitch.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  if (Key = VK_SPACE) or (Key = VK_RETURN) then
    SetChecked(not FChecked);
end;

procedure TSwitch.Paint;
begin
  DrawAll;
end;

procedure TSwitch.SetCaptionOFF(const AValue: String);
begin
  if AValue = FCaptionOFF then exit;
  FCaptionOFF := AValue;
  Invalidate;
end;

procedure TSwitch.SetCaptionON(const AValue: String);
begin
  if AValue = FCaptionON then exit;
  FCaptionON := AValue;
  Invalidate;
end;

procedure TSwitch.SetChecked(const AValue: Boolean);
begin
  if AValue = FChecked then exit;
  FChecked := AValue;
  Invalidate;
  DoChange;
end;

procedure TSwitch.SetColorOFF(const AValue: TColor);
begin
  if AValue = FColorOFF then exit;
  FColorOFF := AValue;
  Invalidate;
end;

procedure TSwitch.SetColorON(const AValue: TColor);
begin
  if AValue = FColorON then exit;
  FColorON := AValue;
  Invalidate;
end;

procedure TSwitch.SetSliderWidth(const AValue: Integer);
begin
  if AValue = FSliderWidth then exit;
  FSliderWidth := AValue;
  Invalidate;
end;

end.

